==> ga4gh.gks.metaschema-0.2.0-py3-none-any.whl: ga4gh.gks.metaschema-0.2.0.data/scripts/y2t.py <==
#!python
"""convert input .yaml to .rst artifacts"""

import os
import sys
import pathlib
from inflector import Inflector
from ga4gh.gks.metaschema.tools.source_proc import YamlSchemaProcessor


i = Inflector()


def resolve_type(class_property_definition):
    if 'type' in class_property_definition:
        if class_property_definition['type'] == 'array':
            return resolve_type(class_property_definition['items'])
        return class_property_definition['type']
    elif '$ref' in class_property_definition:
        ref = class_property_definition['$ref']
        identifier = ref.split('/')[-1]
        if ref.startswith('#'):
            return f':ref:`{identifier}`'
        else:
            return f'`{identifier} <{ref}>`_'
    elif '$refCurie' in class_property_definition:
        return class_property_definition['$refCurie']
    elif 'oneOf' in class_property_definition or 'anyOf' in class_property_definition:
        kw = 'oneOf'
        if 'anyOf' in class_property_definition:
            kw = 'anyOf'
        deprecated_types = class_property_definition.get('deprecated', list())
        resolved_deprecated = list()
        resolved_active = list()
        for property_type in class_property_definition[kw]:
            resolved_type = resolve_type(property_type)
            if property_type in deprecated_types:
                resolved_deprecated.append(resolved_type + f' (deprecated)')
            else:
                resolved_active.append(resolved_type)
        return ' | '.join(resolved_active + resolved_deprecated)
    else:
        raise ValueError(class_property_definition)


def resolve_cardinality(class_property_name, class_property_attributes, class_definition):
    """Resolve class property cardinality from yaml definition"""
    if class_property_name in class_definition.get('required', []):
        min_count = '1'
    elif class_property_name in class_definition.get('heritableRequired', []):
        min_count = '1'
    else:
        min_count = '0'
    if class_property_attributes.get('type') == 'array':
        max_count = class_property_attributes.get('maxItems', 'm')
        min_count = class_property_attributes.get('minItems', 0)
    else:
        max_count = '1'
    return f'{min_count}..{max_count}'


def get_ancestor_with_attributes(class_name, proc):
    if proc.class_is_passthrough(class_name):
        raw_def, proc = proc.get_local_or_inherited_class(class_name, raw=True)
        ancestor = raw_def.get('inherits')
        return get_ancestor_with_attributes(ancestor, proc)
    return class_name


def main(proc_schema, defs_path):
    for class_name, class_definition in proc_schema.defs.items():
        with open(defs_path / (class_name + '.rst'), "w") as f:
            print("**Computational Definition**\n", file=f)
            print(class_definition['description'], file=f)
            if proc_schema.class_is_passthrough(class_name):
                continue
            if 'heritableProperties' in class_definition:
                p = 'heritableProperties'
            elif 'properties' in class_definition:
                p = 'properties'
            elif proc_schema.class_is_primitive(class_name):
                continue
            else:
                raise ValueError(class_name, class_definition)
            ancestor = proc_schema.raw_defs[class_name].get('inherits')
            if ancestor:
                ancestor = get_ancestor_with_attributes(ancestor, proc_schema)
                inheritance = f"\nSome {class_name} attributes are inherited from :ref:`{ancestor}`.\n"
            else:
                inheritance = ""
            print(f"""
    **Information Model**
    {inheritance}
    .. list-table::
       :class: clean-wrap
       :header-rows: 1
       :align: left
       :widths: auto
       
       *  - Field
          - Type
          - Limits
          - Description""", file=f)
            for class_property_name, class_property_attributes in class_definition[p].items():
                print(f"""\
       *  - {class_property_name}
          - {resolve_type(class_property_attributes)}
          - {resolve_cardinality(class_property_name, class_property_attributes, class_definition)}
          - {class_property_attributes.get('description', '')}""", file=f)


if __name__ == "__main__":
    source_file = pathlib.Path(sys.argv[1])

    defs = pathlib.Path.cwd() / 'defs' / str(source_file.stem)[:-7]
    os.makedirs(defs)  # error expected if directory already exists – clear with Make
    p = YamlSchemaProcessor(source_file)
    if p.defs is None:
        exit(0)
    main(p, defs)

==> ga4gh.gks.metaschema-0.3.0b9-py3-none-any.whl: ga4gh.gks.metaschema-0.3.0b9.data/scripts/y2t.py <==
#!python
"""convert input .yaml to .rst artifacts"""

import os
import sys
import pathlib
from ga4gh.gks.metaschema.tools.source_proc import YamlSchemaProcessor


def resolve_type(class_property_definition):
    if 'type' in class_property_definition:
        if class_property_definition['type'] == 'array':
            return resolve_type(class_property_definition['items'])
        return class_property_definition['type']
    elif '$ref' in class_property_definition:
        ref = class_property_definition['$ref']
        identifier = ref.split('/')[-1]
        if ref.startswith('#'):
            return f':ref:`{identifier}`'
        else:
            return f'`{identifier} <{ref}>`_'
    elif '$refCurie' in class_property_definition:
        return class_property_definition['$refCurie']
    elif 'oneOf' in class_property_definition or 'anyOf' in class_property_definition:
        kw = 'oneOf'
        if 'anyOf' in class_property_definition:
            kw = 'anyOf'
        deprecated_types = class_property_definition.get('deprecated', list())
        resolved_deprecated = list()
        resolved_active = list()
        for property_type in class_property_definition[kw]:
            resolved_type = resolve_type(property_type)
            if property_type in deprecated_types:
                resolved_deprecated.append(resolved_type + f' (deprecated)')
            else:
                resolved_active.append(resolved_type)
        return ' | '.join(resolved_active + resolved_deprecated)
    else:
        return "_Not Specified_"


def resolve_cardinality(class_property_name, class_property_attributes, class_definition):
    """Resolve class property cardinality from yaml definition"""
    if class_property_name in class_definition.get('required', []):
        min_count = '1'
    elif class_property_name in class_definition.get('heritableRequired', []):
        min_count = '1'
    else:
        min_count = '0'
    if class_property_attributes.get('type') == 'array':
        max_count = class_property_attributes.get('maxItems', 'm')
        min_count = class_property_attributes.get('minItems', 0)
    else:
        max_count = '1'
    return f'{min_count}..{max_count}'


def get_ancestor_with_attributes(class_name, proc):
    if proc.class_is_passthrough(class_name):
        raw_def, proc = proc.get_local_or_inherited_class(class_name, raw=True)
        ancestor = raw_def.get('inherits')
        return get_ancestor_with_attributes(ancestor, proc)
    return class_name


def main(proc_schema):
    for class_name, class_definition in proc_schema.defs.items():
        with open(proc_schema.def_fp / (class_name + '.rst'), "w") as f:
            print("**Computational Definition**\n", file=f)
            print(class_definition['description'], file=f)
            if proc_schema.class_is_passthrough(class_name):
                continue
            if 'heritableProperties' in class_definition:
                p = 'heritableProperties'
            elif 'properties' in class_definition:
                p = 'properties'
            elif proc_schema.class_is_primitive(class_name):
                continue
            else:
                raise ValueError(class_name, class_definition)
            ancestor = proc_schema.raw_defs[class_name].get('inherits')
            if ancestor:
                ancestor = get_ancestor_with_attributes(ancestor, proc_schema)
                inheritance = f"\nSome {class_name} attributes are inherited from :ref:`{ancestor}`.\n"
            else:
                inheritance = ""
            print(f"""
    **Information Model**
    {inheritance}
    .. list-table::
       :class: clean-wrap
       :header-rows: 1
       :align: left
       :widths: auto
       
       *  - Field
          - Type
          - Limits
          - Description""", file=f)
            for class_property_name, class_property_attributes in class_definition[p].items():
                print(f"""\
       *  - {class_property_name}
          - {resolve_type(class_property_attributes)}
          - {resolve_cardinality(class_property_name, class_property_attributes, class_definition)}
          - {class_property_attributes.get('description', '')}""", file=f)


if __name__ == "__main__":
    source_file = pathlib.Path(sys.argv[1])
    p = YamlSchemaProcessor(source_file)
    os.makedirs(p.def_fp, exist_ok=True)
    if p.defs is None:
        exit(0)
    main(p)
